/**
    (C) Copyright 2012 Dominik Wagenfuehr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "StrategyFactory.hh"
#include "BaseStrategy.hh"

// Gibt Instanz der Klasse zurueck.
StrategyFactory& StrategyFactory::getInstance()
{
    // erstelle einmalig die Klasse
    static StrategyFactory factory;
    return factory;
}

// Registriert eine Strategie.
bool StrategyFactory::registerStrategy( const std::string& id, BaseStrategy* baseP )
{
    bool retValue = false;

    if ( baseP )
    {
        if ( mStrategies.count( id ) == 0 )
        {
            mStrategies.insert( std::pair<std::string,BaseStrategy*>(id, baseP) );
        }
        // else: Strategie bereits eingetragen
    }
    // else: Zeiger ist 0

    return retValue;
}

// Erstellt eine bestimmte Strategie.
BaseStrategy* StrategyFactory::create( const std::string& id, IStrategyContext* contextP  )
{
    BaseStrategy* strategyP = 0;

    // Suche Strategie in Array.
    if ( mStrategies.count( id ) != 0 )
    {
        // Achtung: Wir duerfen nicht einfach den Pointer
        // kopieren, da wir sonst immer das gleiche Objekt
        // nach aussen geben wuerden. Daher muessen wir ein
        // komplett neues Objekt erstellen.
        if ( mStrategies[id] )
        {
            strategyP = mStrategies[id]->copy( contextP );
        }
    }

    return strategyP;
}
