/**
    (C) Copyright 2012 Dominik Wagenfuehr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "VariableStrategy.hh"
#include "IStrategyContext.hh"
#include "StrategyFactory.hh"

// Konstruktor.
VariableStrategy::VariableStrategy( IStrategyContext* contextP )
  : BaseStrategy(contextP), mNumAcceptInRow(0), mNumDeclineInRow(0)
{
}

// Copy Konstruktor.
VariableStrategy::VariableStrategy( const VariableStrategy& object, IStrategyContext* contextP )
  : BaseStrategy(contextP),
    mNumAcceptInRow(object.mNumAcceptInRow), mNumDeclineInRow(object.mNumDeclineInRow)
{
}

// Akzeptiert oder lehnt ein Angebot ab.
bool VariableStrategy::acceptOffer( const int value )
{
    bool retValue = false;

    if ( value >= 200 )
    {
        retValue = true;

        // Wir merken uns, wie oft wir ein großes Angebot
        // angenommen haben.
        if ( value > 700 )
        {
            mNumAcceptInRow++;
        }
        else
        {
            mNumAcceptInRow = 0;
        }
        mNumDeclineInRow = 0;
    }
    else
    {
        // Wert ist zu klein, das merken wir uns
        retValue = false;
        mNumDeclineInRow++;
        mNumAcceptInRow = 0;
    }

    if ( mNumAcceptInRow >= 3 )
    {
        // Immer annehmen, wenn dreimal angenommen wurde.
        if ( mStrategyContextP )
        {
            BaseStrategy* strategyP =
                StrategyFactory::getInstance().create( "accept", mStrategyContextP );
            mStrategyContextP->setStrategy( strategyP );
        }
    }
    else if ( mNumDeclineInRow >= 3 )
    {
        // Immer ablehnen, wenn dreimal abgelehnt wurde.
        if ( mStrategyContextP )
        {
            BaseStrategy* strategyP =
                StrategyFactory::getInstance().create( "decline", mStrategyContextP );
            mStrategyContextP->setStrategy( strategyP );
        }
    }

    return retValue;
}

// Erstellt eine Kopie des Elements.
BaseStrategy* VariableStrategy::copy( IStrategyContext* contextP ) const
{
    return (BaseStrategy*)(new VariableStrategy(*this, contextP));
}
